package org.xbib.interlibrary.http;

import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.function.Consumer;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.xbib.datastructures.tiny.TinyMap;
import org.xbib.net.SocketConfig;
import org.xbib.net.http.HttpHeaders;
import org.xbib.net.http.HttpVersion;
import org.xbib.net.http.client.ExceptionListener;
import org.xbib.net.http.client.HttpResponse;
import org.xbib.net.http.client.ResponseListener;
import org.xbib.net.http.client.TimeoutListener;
import org.xbib.net.http.client.netty.HttpRequest;
import org.xbib.net.http.client.netty.NettyHttpClient;
import org.xbib.net.http.client.netty.NettyHttpClientConfig;
import org.xbib.net.http.client.netty.secure.NettyHttpsClientConfig;
import org.xbib.settings.Settings;

public class HttpHelper {

    private static final Logger logger = Logger.getLogger(HttpHelper.class.getName());

    private NettyHttpClient client;

    private final AtomicBoolean closed;

    public HttpHelper() {
        this.closed =  new AtomicBoolean(true);
    }

    public void open(Settings settings) throws IOException {
        if (closed.compareAndSet(true, false)) {
            if (client == null) {
                client = newClient(settings);
            }
        }
    }

    public void close() throws IOException {
        if (closed.compareAndSet(false, true)) {
            if (client != null) {
                client.close();
                client = null;
            }
        }
    }

    public void post(String url,
                     String body,
                     Consumer<TinyMap.Builder<String, Object>> parameterCompletion,
                     Consumer<HttpHeaders> headersConsumer,
                     ResponseListener<HttpResponse> responseListener,
                     ExceptionListener exceptionListener,
                     TimeoutListener timeoutListener) throws IOException {
        HttpHeaders httpHeaders = new HttpHeaders();
        if (headersConsumer != null) {
            headersConsumer.accept(httpHeaders);
        }
        TinyMap.Builder<String, Object> parameters = TinyMap.builder();
        if (parameterCompletion != null) {
            parameterCompletion.accept(parameters);
        }
        HttpRequest request = HttpRequest.post()
                .setVersion(HttpVersion.HTTP_1_1)
                .setURL(url)
                .setHeaders(httpHeaders)
                .setParameters(parameters.build())
                .setBody(body != null ? StandardCharsets.UTF_8.encode(body) : null)
                .setResponseListener(responseListener)
                .setExceptionListener(exceptionListener)
                .setTimeoutListener(timeoutListener, client.getClientConfig().getSocketConfig().getReadTimeoutMillis())
                .build();
        logger.log(Level.FINE, "executing HTTP POST request " + request);
        client.execute(request).get().close();
    }

    public void get(String url,
                    Consumer<TinyMap.Builder<String, Object>> parameterCompletion,
                    Consumer<HttpHeaders> headersConsumer,
                    ResponseListener<HttpResponse> responseListener,
                    ExceptionListener exceptionListener,
                    TimeoutListener timeoutListener) throws IOException {
        HttpHeaders httpHeaders = new HttpHeaders();
        if (headersConsumer != null) {
            headersConsumer.accept(httpHeaders);
        }
        TinyMap.Builder<String, Object> parameters = TinyMap.builder();
        if (parameterCompletion != null) {
            parameterCompletion.accept(parameters);
        }
        HttpRequest request = HttpRequest.get()
                .setVersion(HttpVersion.HTTP_1_1)
                .setURL(url)
                .setHeaders(httpHeaders)
                .setParameters(parameters.build())
                .setResponseListener(responseListener)
                .setExceptionListener(exceptionListener)
                .setTimeoutListener(timeoutListener, client.getClientConfig().getSocketConfig().getReadTimeoutMillis())
                .build();
        logger.log(Level.FINE, "executing HTTP GET request " + request);
        client.execute(request).get().close();
    }

    private NettyHttpClient newClient(Settings settings) throws IOException {
        SocketConfig socketConfig = new SocketConfig();
        socketConfig.setConnectTimeoutMillis(settings.getAsInt("timeout.connect", 5000));
        socketConfig.setReadTimeoutMillis(settings.getAsInt("timeout.read", 15000));
        socketConfig.setSocketTimeoutMillis(settings.getAsInt("timeout.socket", 15000));
        NettyHttpClientConfig config = new NettyHttpsClientConfig();
        config.setSocketConfig(socketConfig);
        return NettyHttpClient.builder()
                .setConfig(config)
                .build();
    }
}
